import logging
import sys
from dataclasses import dataclass
from typing import Dict, Optional

from lyric_task import Language, LanguageType

from ._py_lyric import PyConfig, PyLocalEnvironmentConfig, PyLyric
from .config import DEFAULT_WORKER_PATH
from .py_lyric import Lyric

logger = logging.getLogger(__name__)


@dataclass
class CodeResult:
    """Container for code execution results

    Attributes:
        exit_code: Exit status of the code execution (0 for success)
        stdout: Standard output from the execution
        stderr: Standard error output from the execution
        output: Optional dictionary containing additional execution data
    """

    exit_code: int
    stdout: str
    stderr: str
    output: Optional[Dict] = None

    @property
    def logs(self) -> str:
        """Combine stdout and stderr into a single log string

        Returns:
            Combined log output with stderr appended after stdout if present
        """
        logs = self.stdout
        if self.stderr:
            logs += f"\n{self.stderr}"
        return logs


class DefaultLyricDriver:
    """A simplified driver for the Lyric execution environment

    This class provides a higher-level interface for code execution across different
    programming languages, handling worker management and environment configuration.

    Attributes:
        py_config: Configuration for the Python Lyric driver
        _lyric: Instance of the core Lyric execution engine
    """

    def __init__(
        self,
        host: str = "127.0.0.1",
        port: int = 15670,
        maximum_workers: int = 20,
        minimum_workers: int = 1,
        log_level: str = "ERROR",
        eventloop_worker_threads: int = 10,
    ):
        """Initialize the DefaultLyricDriver

        Args:
            host: Host address for the driver (default: "127.0.0.1")
            port: Port number for the driver (default: 15670)
            maximum_workers: Maximum number of concurrent worker processes (default: 20)
            minimum_workers: Minimum number of worker processes to maintain (default: 1)
            log_level: Logging level for the driver (default: "ERROR")
            eventloop_worker_threads: Number of worker threads for the event loop (default: 10)
        """
        # Handle localhost alias
        if host == "localhost":
            host = "127.0.0.1"

        # Configure host-specific settings
        if host == "127.0.0.1":
            public_host = "127.0.0.1"
            worker_host = " --host 127.0.0.1"
            driver_public_host = "--public-host 127.0.0.1"
        else:
            public_host = None
            worker_host = ""
            driver_public_host = ""

        # Construct worker command with appropriate host settings
        default_worker = (
            f"{sys.executable} {DEFAULT_WORKER_PATH} {worker_host} {driver_public_host}"
        )

        # Initialize driver configuration
        self.py_config = PyConfig(
            host=host,
            port=port,
            is_driver=True,
            public_host=public_host,
            worker_port_start=15671,
            worker_port_end=16670,
            maximum_workers=maximum_workers,
            minimum_workers=minimum_workers,
            worker_start_commands={
                "PYTHON": default_worker,
                "RUST": default_worker,
                "JAVASCRIPT": default_worker,
            },
            eventloop_worker_threads=eventloop_worker_threads,
            log_level=log_level,
        )

        # Initialize default environment configuration
        default_local_env = PyLocalEnvironmentConfig(
            envs={
                "LYRIC_CORE_LOG_ANSICOLOR": "false",
                "LYRIC_CORE_LOG_LEVEL": log_level,
            }
        )

        # Create Lyric instance
        self._lyric = Lyric(
            PyLyric(self.py_config), default_local_env=default_local_env
        )

    @property
    def lyric(self) -> Lyric:
        """Get the underlying Lyric instance."""
        return self._lyric

    def start(self):
        """Start the Lyric driver."""
        self._lyric.start_driver()

    async def exec(
        self,
        code: str,
        lang: LanguageType = Language.PYTHON,
        worker_name: Optional[str] = None,
        decode: bool = True,
    ) -> CodeResult:
        """Execute code in the specified language

        Args:
            code: Source code to execute
            lang: Programming language of the code (default: Python)
            worker_name: Optional name of specific worker to use
            decode: Whether to decode the execution result (default: True)

        Returns:
            CodeResult containing execution status and output
        """
        try:
            res = await self._lyric.exec(
                code, lang=lang, worker_name=worker_name, decode=decode
            )
            stdout = res.get("stdout", "")
            stderr = res.get("stderr", "")
            exit_code = res.get("exit_code", 1)
        except Exception as e:
            stdout = ""
            stderr = str(e)
            exit_code = 1
        return CodeResult(exit_code=exit_code, stdout=stdout, stderr=stderr)

    async def exec1(
        self,
        code: str,
        input_bytes: bytes,
        call_name: str,
        encode: bool = True,
        decode: bool = True,
        lang: LanguageType = Language.PYTHON,
        worker_name: Optional[str] = None,
    ) -> CodeResult:
        """Execute code with input data and a specific function call

        Args:
            code: Source code to execute
            input_bytes: Input data as bytes
            call_name: Name of the function to call in the code
            encode: Whether to encode the input (default: True)
            decode: Whether to decode the result (default: True)
            lang: Programming language of the code (default: Python)
            worker_name: Optional name of specific worker to use

        Returns:
            CodeResult containing execution status, output and additional data
        """
        output = None
        try:
            res, output = await self._lyric.exec1(
                code,
                input_bytes,
                call_name,
                encode=encode,
                decode=decode,
                lang=lang,
                worker_name=worker_name,
            )
            stdout = res.get("stdout", "")
            stderr = res.get("stderr", "")
            exit_code = res.get("exit_code", 1)
        except Exception as e:
            stdout = ""
            stderr = str(e)
            exit_code = 1
        return CodeResult(
            exit_code=exit_code, stdout=stdout, stderr=stderr, output=output
        )

    def stop(self):
        """Stop the Lyric driver and cleanup resources."""
        self._lyric.stop()



import asyncio
import json
import logging
import uuid
from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum
from typing import Any, Callable, Dict, List, Optional, Tuple, Union

from lyric.task import TaskInfo
from lyric_task import BaseTaskSpec, Language, LanguageType, WasmTaskSpec

from ._py_lyric import (
    PyDockerEnvironmentConfig,
    PyDriverConfig,
    PyEnvironmentConfig,
    PyLocalEnvironmentConfig,
    PyLyric,
    PyTaskHandle,
)

logger = logging.getLogger(__name__)


class ExecEnvType(str, Enum):
    LOCAL = "local"
    DOCKER = "docker"


EXEC_ENV = Union[
    str,
    ExecEnvType,
    PyLocalEnvironmentConfig,
    PyDockerEnvironmentConfig,
    PyEnvironmentConfig,
]


@dataclass
class WorkerInfo:
    """Basic information about a worker"""

    name: str  # Unique identifier for the worker, e.g. "python_worker"
    lang: Language  # Language type supported by the worker
    loader: "WorkerLoader"  # Worker's loader instance


class WorkerLoader(ABC):
    """Abstract base class for worker loaders"""

    async def load_worker(
        self,
        lyric: PyLyric,
        task_id: Optional[str] = None,
        task_name: Optional[str] = None,
        config: Optional[PyEnvironmentConfig] = None,
    ) -> PyTaskHandle:
        task_spec = self._task_spec()
        task_id = task_id or f"{str(uuid.uuid4())}"
        task_name = task_name or task_id
        task_info = TaskInfo.from_task(task_name, task_id, 3, task_spec)
        return await lyric.submit_task(task_info.to_core(), environment_config=config)

    @abstractmethod
    def _task_spec(self) -> BaseTaskSpec:
        """Get the task spec for the worker"""


class PythonWorkerLoader(WorkerLoader):
    def _task_spec(self) -> BaseTaskSpec:
        try:
            from lyric_py_worker import PythonWasmTaskSpec
        except ImportError:
            raise ImportError(
                "lyric_py_worker is not installed. Please install it with: pip install lyric-py-worker"
            )
        return PythonWasmTaskSpec()


class JavaScriptWorkerLoader(WorkerLoader):
    def _task_spec(self) -> BaseTaskSpec:
        try:
            from lyric_js_worker import JavaScriptWasmTaskSpec
        except ImportError:
            raise ImportError(
                "lyric_js_worker is not installed. Please install it with: pip install lyric-js-worker"
            )
        return JavaScriptWasmTaskSpec()


class RawWasmWorkerLoader(WorkerLoader):
    def __init__(self, wasm_path: str, lang: LanguageType = Language.PYTHON):
        self._wasm_path = wasm_path
        self._lang = lang

    def _task_spec(self) -> BaseTaskSpec:
        return WasmTaskSpec(self._wasm_path, self._lang)


@dataclass
class HandleItem:
    """Container for worker handle information and instance"""

    worker_info: WorkerInfo
    handle: PyTaskHandle
    environment_config: Optional[PyEnvironmentConfig] = None

    @property
    def uid(self) -> str:
        """Generate unique identifier for the handle"""
        config_id = (
            self.environment_config.env_id() if self.environment_config else "none"
        )
        return f"{self.worker_info.name}_{config_id}"


class HandleManager:
    """Manager for worker handles"""

    def __init__(self, lyric: PyLyric):
        self._lyric = lyric
        self._lock = asyncio.Lock()
        self._handles: Dict[str, HandleItem] = {}

    def get_worker_handles(self, worker_name: str) -> List[HandleItem]:
        """Get all handles for specified worker name"""
        return [h for h in self._handles.values() if h.worker_info.name == worker_name]

    def get_lang_handles(self, lang: Language) -> List[HandleItem]:
        """Get all handles for specified language"""
        return [h for h in self._handles.values() if h.worker_info.lang == lang]

    async def get_or_create_handler(
        self,
        worker_info: WorkerInfo,
        environment_config: Optional[PyEnvironmentConfig] = None,
        ignore_load_error: bool = False,
    ) -> Optional[HandleItem]:
        """Get existing handle or create new one"""
        async with self._lock:
            handle_item = HandleItem(worker_info, None, environment_config)
            uid = handle_item.uid

            if uid in self._handles:
                return self._handles[uid]

            try:
                handle = await worker_info.loader.load_worker(
                    self._lyric, config=environment_config
                )
                handle_item.handle = handle
                self._handles[uid] = handle_item
                return handle_item
            except Exception as e:
                if ignore_load_error:
                    logger.warning(f"Failed to load worker {worker_info.name}: {e}")
                    return None
                raise e


class ConfigurationManager:
    """Manager for environment configurations"""

    def __init__(
        self,
        default_local_env: Optional[PyLocalEnvironmentConfig] = None,
        default_docker_config: Optional[PyDockerEnvironmentConfig] = None,
    ):
        envs = {
            "LYRIC_CORE_LOG_ANSICOLOR": "false",
            "LYRIC_CORE_LOG_LEVEL": "ERROR",
        }
        self._default_local_config = default_local_env or PyLocalEnvironmentConfig(
            envs=envs
        )
        self._default_docker_config = (
            default_docker_config
            or PyDockerEnvironmentConfig(
                image="py-lyric-base-alpine:latest",
                mounts=[(self._get_base_lyric_dir(), "/app")],
                envs=envs,
            )
        )

    @staticmethod
    def _get_base_lyric_dir() -> str:
        from lyric import BASE_LYRIC_DIR

        return BASE_LYRIC_DIR

    def create_worker_config(self, worker_name: str) -> PyEnvironmentConfig:
        """Create default configuration for specified worker"""
        base_config = PyEnvironmentConfig(local=self._default_local_config)
        return base_config.clone_new(worker_name)

    def get_environment_config(
        self, exec_env: Optional[EXEC_ENV], worker_name: str
    ) -> PyEnvironmentConfig:
        """Get environment configuration"""
        env_config: Optional[PyEnvironmentConfig] = None
        if isinstance(exec_env, (ExecEnvType, str)):
            if exec_env == ExecEnvType.DOCKER:
                env_config = PyEnvironmentConfig(docker=self._default_docker_config)
            elif exec_env == ExecEnvType.LOCAL:
                env_config = PyEnvironmentConfig(local=self._default_local_config)
        elif isinstance(exec_env, PyEnvironmentConfig):
            env_config = exec_env
        elif isinstance(exec_env, PyLocalEnvironmentConfig):
            env_config = PyEnvironmentConfig(local=exec_env)
        elif isinstance(exec_env, PyDockerEnvironmentConfig):
            env_config = PyEnvironmentConfig(docker=exec_env)
        if not env_config:
            env_config = self.create_worker_config(worker_name)
        return env_config.clone_new(worker_name)


class Lyric:
    """Main Lyric class providing API interfaces"""

    DEFAULT_WORKERS = [
        WorkerInfo("python_worker", Language.PYTHON, PythonWorkerLoader()),
        WorkerInfo("javascript_worker", Language.JAVASCRIPT, JavaScriptWorkerLoader()),
    ]

    def __init__(
        self,
        pl: PyLyric,
        default_local_env: Optional[PyLocalEnvironmentConfig] = None,
        default_docker_config: Optional[PyDockerEnvironmentConfig] = None,
    ):
        self._pl = pl
        self._handle_manager = HandleManager(pl)
        self._config_manager = ConfigurationManager(
            default_local_env, default_docker_config
        )
        self._workers: Dict[str, WorkerInfo] = {w.name: w for w in self.DEFAULT_WORKERS}

    def start_driver(self):
        """Start the driver"""
        self._pl.start_driver(PyDriverConfig())

    def register_worker(
        self, name: str, lang: LanguageType, loader: WorkerLoader
    ) -> None:
        """Register a new worker

        Args:
            name: Unique name for the worker
            lang: Language type supported by the worker
            loader: Worker loader instance
        """
        if name in self._workers:
            raise ValueError(f"Worker '{name}' already exists")

        lang_instance = Language.parse(lang)
        worker_info = WorkerInfo(name, lang_instance, loader)
        self._workers[name] = worker_info

    async def load_worker(
        self, worker_name: str, exec_env: Optional[EXEC_ENV] = None
    ) -> None:
        """Load specified worker

        Args:
            worker_name: Name of the worker to load
            exec_env: Optional execution environment configuration
        """
        if worker_name not in self._workers:
            raise ValueError(f"Worker '{worker_name}' not found")

        worker_info = self._workers[worker_name]
        config = self._config_manager.get_environment_config(exec_env, worker_name)
        await self._handle_manager.get_or_create_handler(worker_info, config, True)

    async def load_default_workers(self) -> None:
        """Load all default workers"""
        for worker_info in self.DEFAULT_WORKERS:
            config = self._config_manager.create_worker_config(worker_info.name)
            await self._handle_manager.get_or_create_handler(worker_info, config, True)

    def get_worker_info(self, worker_name: str) -> Optional[WorkerInfo]:
        """Get worker information"""
        return self._workers.get(worker_name)

    def list_workers(self) -> List[WorkerInfo]:
        """List all registered workers"""
        return list(self._workers.values())

    async def _get_handler(
        self,
        lang: Language,
        worker_name: Optional[str] = None,
        exec_env: Optional[EXEC_ENV] = None,
    ) -> HandleItem:
        """Get handle that meets the conditions"""
        if worker_name:
            if worker_name not in self._workers:
                raise ValueError(f"Worker '{worker_name}' not found")
            worker_info = self._workers[worker_name]
            if worker_info.lang != lang:
                raise ValueError(
                    f"Worker '{worker_name}' does not support language {lang}"
                )
        else:
            # Use default worker
            worker_info = next(
                (w for w in self.DEFAULT_WORKERS if w.lang == lang), None
            )
            if not worker_info:
                raise ValueError(f"No default worker found for language {lang}")

        config = self._config_manager.get_environment_config(exec_env, worker_info.name)
        handle = await self._handle_manager.get_or_create_handler(worker_info, config)

        if not handle:
            raise RuntimeError(f"Failed to get handle for worker '{worker_info.name}'")
        return handle

    async def exec(
        self,
        code: str,
        lang: LanguageType = Language.PYTHON,
        worker_name: Optional[str] = None,
        decode: bool = True,
        exec_env: Optional[EXEC_ENV] = None,
    ) -> Union[Dict[str, Any], bytes]:
        """Execute code

        Args:
            code: Code to execute
            lang: Language type of the code
            worker_name: Optional worker name to use
            decode: Whether to decode the result
            exec_env: Optional execution environment configuration
        """
        lang_instance = Language.parse(lang)
        handle = await self._get_handler(lang_instance, worker_name, exec_env)

        script_res = await handle.handle.exec(lang_instance.name, code, decode=decode)

        try:
            encoded = script_res.data
            if decode:
                json_str = bytes(encoded).decode("utf-8")
                return json.loads(json_str)
            return encoded
        except Exception as e:
            logger.error(f"Failed to parse the result: {e}")
            raise e

    async def exec1(
        self,
        code: str,
        input_bytes: bytes,
        call_name: str,
        encode: bool = True,
        decode: bool = True,
        lang: LanguageType = Language.PYTHON,
        worker_name: Optional[str] = None,
        exec_env: Optional[EXEC_ENV] = None,
    ) -> Union[Tuple[Dict[str, Any], Dict[str, Any]], Tuple[bytes, bytes]]:
        """Execute code with input

        Args:
            code: Code to execute
            input_bytes: Input data
            call_name: Function name to call
            encode: Whether to encode the input
            decode: Whether to decode the result
            lang: Language type of the code
            worker_name: Optional worker name to use
            exec_env: Optional execution environment configuration
        """
        lang_instance = Language.parse(lang)
        handle = await self._get_handler(lang_instance, worker_name, exec_env)

        script_res = await handle.handle.exec1(
            lang_instance.name,
            code,
            call_name=call_name,
            input=input_bytes,
            encode=encode,
            decode=decode,
        )

        res_bytes = bytes(script_res[0].data)
        output_bytes = bytes(script_res[1].data)

        try:
            if decode:
                return (
                    json.loads(res_bytes.decode("utf-8")),
                    json.loads(output_bytes.decode("utf-8")),
                )
            return res_bytes, output_bytes
        except Exception as e:
            logger.error(f"Failed to parse the result: {e}")
            raise e

    def stop(self):
        """Stop Lyric instance"""
        self._pl.stop()

    def join(self):
        """Wait for all tasks to complete"""
        self._pl.join()



import uuid
from enum import Enum
from typing import Any, Optional

import cloudpickle
from lyric_task import BaseTaskSpec, ExecutableTaskSpec, WasmTaskSpec

from ._py_lyric import (
    PyDataObject,
    PyExecutionUnit,
    PyTaskInfo,
    PyTaskOutputObject,
    PyTaskStateInfo,
)


class Language(Enum):
    PYTHON = 0
    NODEJS = 1
    SHELL = 2


class DataFormat(Enum):
    RAW = 0
    PICKLE = 1
    MSGPACK = 2
    PROTOBUF = 3

    @classmethod
    def from_int(cls, value: int):
        return cls(value)


class TaskOutputObject:
    def __init__(self, output: Any, stdout: str, stderr: str):
        self.data = output
        self.stdout = stdout
        self.stderr = stderr


def wrapper_task_output(
    output: Any, stdout: str | None, stderr: str | None
) -> PyTaskOutputObject:
    object_id = str(uuid.uuid4())
    code_data = cloudpickle.dumps(output)
    code_object = PyDataObject(
        object_id=object_id, format=DataFormat.PICKLE.value, data=code_data
    )
    return PyTaskOutputObject(code_object, stdout=stdout or "", stderr=stderr or "")


def unwrapper_task_output(output: PyTaskStateInfo) -> TaskOutputObject:
    data = output.output
    code_format = DataFormat(data.format)
    code_data = data.data
    if code_format == DataFormat.PICKLE:
        return TaskOutputObject(
            cloudpickle.loads(bytes(code_data)), output.stdout, output.stderr
        )
    else:
        raise ValueError(f"Unsupported data format: {code_format}")


def unwrap_task_output_data(output: PyDataObject) -> Any:
    code_format = DataFormat(output.format)
    code_data = output.data
    if code_format == DataFormat.PICKLE:
        return cloudpickle.loads(bytes(code_data))
    else:
        raise ValueError(f"Unsupported data format: {code_format}")


class TaskInfo:
    def __init__(
        self,
        name: str,
        task_id: str,
        language: int,
        exec_mode: int,
        exec_unit: Optional[PyExecutionUnit] = None,
        input: Optional[PyDataObject] = None,
        streaming_result: bool = False,
    ):
        self.name = name
        self.task_id = task_id
        self.language = language
        self.exec_mode = exec_mode
        self.exec_unit = exec_unit
        self.input = input
        self.streaming_result = streaming_result

    @classmethod
    def from_task(
        cls,
        name: str,
        task_id: str,
        exec_mode: int,
        task: BaseTaskSpec,
        input: Any = None,
    ):
        exec_unit = task.to_execution_unit()
        code_object = PyDataObject(
            object_id=exec_unit.code.object_id,
            format=exec_unit.code.format,
            data=exec_unit.code.data,
        )
        py_exec_unit = PyExecutionUnit(
            unit_id=exec_unit.unit_id, language=exec_unit.language, code=code_object
        )
        if input:
            input = cls.serialize_task(input)
        return cls(
            name=name,
            task_id=task_id,
            language=Language.PYTHON.value,
            exec_mode=exec_mode,
            exec_unit=py_exec_unit,
            input=input,
            streaming_result=task.streaming_result,
        )

    @classmethod
    def serialize_task(cls, task: Any, is_serialization=False) -> PyDataObject:
        data = task if is_serialization else cloudpickle.dumps(task)
        return PyDataObject(
            object_id=str(uuid.uuid4()),
            format=DataFormat.PICKLE.value,
            data=data,
        )

    @classmethod
    def from_core(cls, core_task_info: PyTaskInfo):
        return cls(
            task_id=core_task_info.task_id,
            name=core_task_info.name,
            language=core_task_info.language,
            exec_mode=core_task_info.exec_mode,
            exec_unit=core_task_info.exec_unit,
            input=core_task_info.input,
        )

    def to_core(self) -> PyTaskInfo:
        return PyTaskInfo(
            task_id=self.task_id,
            name=self.name,
            language=self.language,
            exec_mode=self.exec_mode,
            exec_unit=self.exec_unit,
            input=self.input,
            streaming_result=self.streaming_result,
        )

    def to_task(self) -> BaseTaskSpec:
        if not self.exec_unit:
            raise ValueError("No execution unit provided")
        if self.language != Language.PYTHON.value:
            raise ValueError("Only Python tasks are supported on Python worker")
        code_object = self.exec_unit.code
        if not code_object:
            raise ValueError("No code object provided")
        code_format = DataFormat(code_object.format)
        code_data = code_object.data
        if not code_data:
            raise ValueError("No code data provided")
        code_data = bytes(code_data)
        if code_format == DataFormat.RAW:
            str_code = code_data.decode("utf-8")
            return NormalCodeTask(str_code)
        elif code_format == DataFormat.PICKLE:
            # TODO: Support passing input data
            return cloudpickle.loads(code_data)
        else:
            raise ValueError(f"Unsupported data format: {self.exec_unit.data_format}")

    def __str__(self):
        return f"TaskInfo(name={self.name}, task_id={self.task_id}, language={self.language}, exec_mode={self.exec_mode})"



import argparse
import logging

from lyric import PyConfig, PyLyric, PyTaskInfo, PyWorkerConfig, from_python_iterator
from lyric.task import TaskInfo, wrapper_task_output
from lyric_task.log import IOCapture, capture_iterator_output
from lyric_task.utils import (
    AsyncToSyncIterator,
    get_or_create_event_loop,
    is_async_iterator,
    is_iterator,
)

logger = logging.getLogger("lyric")


def on_message(msg: PyTaskInfo):
    task_info = TaskInfo.from_core(msg)
    task = task_info.to_task()
    logger.info(f"Received message on python worker: {task_info}")

    with IOCapture() as capture:
        result = task()
        stdout, stderr = capture.get_output()
    if is_iterator(result):
        return from_python_iterator(
            capture_iterator_output(result, wrapper_task_output)
        )
    elif is_async_iterator(result):
        loop = get_or_create_event_loop()
        new_iter = iter(AsyncToSyncIterator(result, loop))
        return from_python_iterator(
            capture_iterator_output(new_iter, wrapper_task_output)
        )
    else:
        return wrapper_task_output(result, stdout, stderr)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("--host", type=str, default=None, help="Host")
    parser.add_argument("--port", type=int, default=None, help="Port")
    parser.add_argument(
        "--public-host",
        type=str,
        default=None,
        help="Public host to register to driver",
    )
    parser.add_argument(
        "--node-id", type=str, default=None, help="Node ID of the worker"
    )
    parser.add_argument(
        "--driver-address",
        type=str,
        default=None,
        help="Driver address to connect to",
    )
    parser.add_argument("--log_file", type=str, default=None)
    parser.add_argument("--network_mode", type=str, default="host")

    args, _ = parser.parse_known_args()
    node_id = args.node_id
    public_host = args.public_host

    py_config = PyConfig(
        host=args.host,
        port=args.port,
        is_driver=False,
        public_host=public_host,
        worker_port_start=15671,
        worker_port_end=16670,
        maximum_workers=10,
        minimum_workers=1,
        node_id=node_id,
    )
    config = PyWorkerConfig(driver_address=args.driver_address)
    pp = PyLyric(py_config)

    pp.start_worker(config)
    pp.set_callback(on_message)
    pp.join()



"""Lyric Python binding.

Lyric: A Rust-powered secure sandbox for multi-language code execution, leveraging WebAssembly to provide
high-performance runtime isolation for AI applications.

This module provides a Python binding for Lyric, which allows you to execute code in multiple languages

Getting started:

1. Install lyric-py:

    .. code-block:: bash

        pip install lyric-py

2. Install default Python webassembly worker:

    .. code-block:: bash

        pip install lyric-py-worker

3. Install default JavaScript webassembly worker:

    .. code-block:: bash

        pip install lyric-js-worker

Examples:
    .. code-block:: python

        import asyncio
        from lyric import DefaultLyricDriver

        lcd = DefaultLyricDriver(host="localhost", log_level="ERROR")
        lcd.start()

        # Load workers(default: Python, JavaScript)
        asyncio.run(lcd.lyric.load_default_workers())

        # Execute Python code
        python_code = \"""
        def add(a, b):
            return a + b
        result = add(1, 2)
        print(result)
        \"""

        py_res = asyncio.run(lcd.exec(python_code, "python"))
        print(py_res)

        # Execute JavaScript code
        js_code = \"""
        console.log('Hello from JavaScript!');
        \"""

        js_res = asyncio.run(lcd.exec(js_code, "javascript"))
        print(js_res)

        # Stop the driver
        lcd.stop()

"""

from ._py_lyric import *
from .config import BASE_LYRIC_DIR, DEFAULT_WORKER_FILE, DEFAULT_WORKER_PATH
from .driver import CodeResult, DefaultLyricDriver
from .py_lyric import Lyric
from .task import TaskInfo

__doc__ = _py_lyric.__doc__
if hasattr(_py_lyric, "__all__"):
    __all__ = _py_lyric.__all__
else:
    __all__ = []

__ALL__ = __all__
__ALL__.extend(
    [
        "BASE_LYRIC_DIR",
        "DEFAULT_WORKER_FILE",
        "DEFAULT_WORKER_PATH",
        "Lyric",
        "TaskInfo",
        "DefaultLyricDriver",
        "CodeResult",
    ]
)
